/*
 * util.h
 *
 * Copyright (C) 2012 Lan Du
 * All rights reserved.
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations
 * under the License.
 *
 * Author: Lan Du (Lan.Du@mq.edu.au)
 *
 */

#ifndef UTIL_H_
#define UTIL_H_

#include <gsl/gsl_vector.h>
#include <gsl/gsl_matrix.h>
#include <math.h>

#define MAX_ITE_RAN 5
#define MAX_DOUBLE 1.7e307
#define MIN_DOUBLE 1.7e-300

typedef struct pair{
    int key;
    double value;
} pair;

/*
 * vector operations
 */
double vget(const gsl_vector*, int);
void vset(gsl_vector*, int, double);
void vinc(gsl_vector*, int, double);
void vprint(gsl_vector*);
void scanf_vector(char*, gsl_vector*);
void save_vector(char*, gsl_vector*);
double vsum(gsl_vector*);
void normalize(gsl_vector* v);

int v_equal(const gsl_vector*, const gsl_vector*);
int v_contain(const gsl_vector* v, double);
int v_non_zero(const gsl_vector* v);
/*
 * matrix operations
 */
double mget(const gsl_matrix* m, int, int);
void mset(gsl_matrix*, int, int, double);
void minc(gsl_matrix*, int, int, double);
void col_sum(gsl_matrix*, gsl_vector*);
void row_sum(gsl_matrix*, gsl_vector*);
double msum( gsl_matrix*);

void mprint(gsl_matrix*);
void scanf_matrix(char*, gsl_matrix*);
void save_matrix(char*, gsl_matrix* m);
void printf_matrix(char*, gsl_matrix*);

/*
 * sampling operations
 */
void initial_rng();
void free_rng();
double next_uniform();
double next_gamma(double a);
int next_discrete_unnormalised(gsl_vector*, double);
int next_discrete_normalised(gsl_vector*);

void sort(gsl_vector*);
double safe_log(double);
double log_sum(double, double);
/*
 * stirling number generation
 */
void make_stirling_table(int, int, float);
double stirling(int, int, float);
void print_stirling_table(char*);
void free_stirling_table();
/*
 * Pochhammer Symbal
 */
double poch_sym(double, double, int);
double log_poch_sym(double, double, int);
#endif /* UTIL_H_ */
